<?php
/**
 * @package         Articles Anywhere
 * @version         13.0.3PRO
 * 
 * @author          Peter van Westen <info@regularlabs.com>
 * @link            http://regularlabs.com
 * @copyright       Copyright © 2023 Regular Labs All Rights Reserved
 * @license         http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 */

namespace RegularLabs\Plugin\System\ArticlesAnywhere\Collection\Filters;

defined('_JEXEC') or die;


use JDatabaseQuery;
use RegularLabs\Library\DB as RL_DB;
use RegularLabs\Plugin\System\ArticlesAnywhere\Collection\DB;

class Categories extends Filter
{
    public function setFilter(JDatabaseQuery $query, $filters = [])
    {
        $exclude = isset($filters[0])
            ? RL_DB::getOperatorFromValue($filters[0]) == '!='
            : false;

        $include_ids = [];
        $exclude_ids = [];

        foreach ($filters as $filter)
        {
            $exclude = RL_DB::getOperatorFromValue($filter) == '!=';
            $filter  = RL_DB::removeOperator($filter);

            $filter_ids = $this->getIds([$filter]);

            foreach ($filter_ids as $filter_id)
            {
                if ($exclude)
                {

                    $exclude_ids[] = $filter_id;
                    continue;
                }
                $include_ids[] = $filter_id;
            }
        }

        $wheres = [];

        if ( ! empty($include_ids))
        {
            $wheres[] = $this->db->quoteName('items.catid') . RL_DB::in($include_ids);
        }

        if ( ! empty($exclude_ids))
        {
            $exclude_ids[0] = '!=' . $exclude_ids[0];
            $wheres[]       = $this->db->quoteName('items.catid') . RL_DB::in($exclude_ids);
        }

        if (empty($wheres))
        {
            $query->where($exclude ? '1' : '0');

            return;
        }

        $query->where($wheres);
    }

    private function getChildIds($ids = [])
    {
        $query = $this->getChildIdsQuery($ids);

        return DB::getResults($query) ?: [];
    }

    private function getChildIdsQuery($parent_ids = [])
    {
        $query = $this->db->getQuery(true)
            ->select($this->db->quoteName('id'))
            ->from($this->config->getTableCategories('categories'))
            ->where($this->config->get('categories_parent_id')
                . RL_DB::in($parent_ids));

        $extension = $this->config->get('categories_extension', false);

        if ($extension)
        {
            $query->where($this->db->quoteName('extension') . ' = ' . $this->db->quote($extension));
        }

        return $query;
    }

    private function getIds($names = [])
    {
        $query = $this->getIdsQuery($names);

        $ids = DB::getResults($query) ?: [];

        $include_children = $this->config->getFiltersIncludeChildren('categories');

        if ( ! $include_children)
        {
            return $ids;
        }

        $max_depth = is_int($include_children) ? $include_children : 20;
        $depth     = 0;

        $child_ids = $ids;

        while ($depth++ < $max_depth && $child_ids = $this->getChildIds($child_ids))
        {
            $ids = array_merge($ids, $child_ids);
        }

        return $ids;
    }

    private function getIdsQuery($names = [])
    {
        $query = $this->db->getQuery(true)
            ->select($this->db->quoteName('id'))
            ->from($this->config->getTableCategories('categories'));

        $extension = $this->config->get('categories_extension', false);

        if ($extension)
        {
            $query->where($this->db->quoteName('extension') . ' = ' . $this->db->quote($extension));
        }

        $this->setFiltersFromNames($query, 'categories', $names);

        $this->setIgnores($query, 'categories', 'categories');

        return $query;
    }
}
