<?php
/**
 * @package         Articles Anywhere
 * @version         13.0.3PRO
 * 
 * @author          Peter van Westen <info@regularlabs.com>
 * @link            http://regularlabs.com
 * @copyright       Copyright © 2023 Regular Labs All Rights Reserved
 * @license         http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 */

namespace RegularLabs\Plugin\System\ArticlesAnywhere\DataGroups\Helpers;

defined('_JEXEC') or die;


use Joomla\CMS\HTML\HTMLHelper as JHtml;
use Joomla\CMS\Layout\LayoutHelper as JLayout;
use Joomla\CMS\Router\Route as JRoute;
use Joomla\Component\Tags\Site\Helper\RouteHelper as JTagsRouteHelper;
use RegularLabs\Library\ArrayHelper as RL_Array;
use RegularLabs\Plugin\System\ArticlesAnywhere\Helpers\Params;

class Tags
{
    public static function getOutput($tags, $separator = ', ', $last_separator = null, $add_links = true)
    {
        $access_levels = Params::getAuthorisedViewLevels();
        $array         = [];

        $add_links ??= true;

        foreach ($tags as $tag)
        {
            if ( ! in_array($tag->access, $access_levels))
            {
                continue;
            }

            $text = htmlspecialchars($tag->title, ENT_COMPAT, 'UTF-8');

            if ( ! $add_links)
            {
                $array[] = $text;
                continue;
            }

            $attribs = [
                'class' => $tag->tag_link_class ?? '',
            ];

            $array[] = JHtml::_(
                'link',
                self::getTagUrl($tag),
                $text,
                $attribs
            );
        }

        return RL_Array::implode(
            $array,
            $separator,
            $last_separator
        );
    }

    public static function getOutputViaLayout($tags, $layout = null)
    {
        $layout_id = Layout::getId($layout, 'joomla.content.tags');

        return JLayout::render($layout_id, $tags);
    }

    public static function getTagAliases($tags)
    {
        if (empty($tags))
        {
            return [];
        }

        $access_levels = Params::getAuthorisedViewLevels();

        $titles = [];

        foreach ($tags as $tag)
        {
            if ( ! in_array($tag->access, $access_levels))
            {
                continue;
            }

            $titles[] = $tag->alias;
        }

        return $titles;
    }

    public static function getTagTitles($tags)
    {
        if (empty($tags))
        {
            return [];
        }

        $access_levels = Params::getAuthorisedViewLevels();

        $titles = [];

        foreach ($tags as $tag)
        {
            if ( ! in_array($tag->access, $access_levels))
            {
                continue;
            }

            $titles[] = htmlspecialchars($tag->title, ENT_COMPAT, 'UTF-8');
        }

        return $titles;
    }

    private static function getTagUrl($tag)
    {
        return JRoute::link('site', JTagsRouteHelper::getTagRoute($tag->tag_id . ':' . $tag->alias));
    }
//
//    public function get($tags, $attributes)
//    {
//        if (isset($attributes->output)
//            && in_array($attributes->output, ['value', 'values', 'raw']))
//        {
//            $array = [];
//
//            foreach ($tags as $tag)
//            {
//                if ( ! in_array($tag->access, $this->access_levels))
//                {
//                    continue;
//                }
//
//                $array[] = $tag->title;
//            }
//
//            return $array;
//        }
//
//        if (
//            JFactory::getApplication()->input->get('option', '') === 'com_finder'
//            && JFactory::getApplication()->input->get('format', '') === 'json'
//        )
//        {
//            // Force normal layout for finder indexing, as the TagsHelperRoute causes errors
//            $attributes->clean = true;
//        }
//
//        if ( ! empty($attributes->strip))
//        {
//            $separator = $attributes->separator ?? $attributes->delimiter ?? ', ';
//
//            return $this->processTagTagsStripped($tags, $separator);
//        }
//
//        if ( ! empty($attributes->clean))
//        {
//            $separator = $attributes->separator ?? $attributes->delimiter ?? ', ';
//
//            return $this->processTagTagsClean($tags, $separator);
//        }
//
//        return $this->processTagTagsLayout($tags);
//    }
//
//    private function processTagTagsClean($tags, $separator = ' ')
//    {
//        require_once JPATH_ROOT . '/components/com_tags/helpers/route.php';
//
//        $html = [];
//
//        foreach ($tags as $tag)
//        {
//            if ( ! in_array($tag->access, $this->access_levels))
//            {
//                continue;
//            }
//
//            $html[] = '<span class="tag-' . $tag->tag_id . '" itemprop="keywords">'
//                . '<a href = "' . JRoute::link('site', JTagsRouteHelper::getTagRoute($tag->tag_id . '-' . $tag->alias)) . '" class="tag_link">'
//                . htmlspecialchars($tag->title, ENT_COMPAT, 'UTF-8')
//                . '</a>'
//                . '</span>';
//        }
//
//        return '<span class="tags">' . implode($separator, $html) . '</span>';
//    }
//
//    private function processTagTagsLayout($tags)
//    {
//        $layout = new JLayoutFile('joomla.content.tags');
//
//        return $layout->render($tags);
//    }
//
//    private function processTagTagsStripped($tags, $separator = ', ')
//    {
//        $html = [];
//
//        foreach ($tags as $tag)
//        {
//            if ( ! in_array($tag->access, $this->access_levels))
//            {
//                continue;
//            }
//
//            $html[] = htmlspecialchars($tag->title, ENT_COMPAT, 'UTF-8');
//        }
//
//        return implode($separator, $html);
//    }

}
