<?php
/**
 * @package         Articles Anywhere
 * @version         13.0.3PRO
 * 
 * @author          Peter van Westen <info@regularlabs.com>
 * @link            http://regularlabs.com
 * @copyright       Copyright © 2023 Regular Labs All Rights Reserved
 * @license         http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 */

namespace RegularLabs\Plugin\System\ArticlesAnywhere\DataGroups\Helpers;

defined('_JEXEC') or die;

use RegularLabs\Library\Image as RL_Image;
use RegularLabs\Plugin\System\ArticlesAnywhere\Helpers\Params;

class VideoYoutube extends VideoObject
{
    static $regex                 = '(?:youtube\.com\/watch\?v=|youtu\.be/|youtube\.com/embed/)(?<id>[0-9a-z_-]+)(?<query>.*)';
    static $url_prefix            = 'https://www.youtube.com/watch?v=';
    static $url_prefix_embed      = 'https://www.youtube.com/embed/';
    static $url_prefix_no_cookies = 'https://www.youtube-nocookie.com/embed/';
    static $url_prefix_short      = 'https://youtu.be/';

    public function __construct($iframe, $url)
    {
        $params = Params::get();

        if ($params->youtube_embed_url === 'no_cookies')
        {
            self::$url_prefix_embed = self::$url_prefix_no_cookies;
        }

        parent::__construct($iframe, $url);
    }

    protected function getEmbedUrl()
    {
        $url = parent::getEmbedUrl();

        if (strpos($url, 'wmode=transparent') !== false)
        {
            return $url;
        }

        $url .= (strpos($url, '?') === false ? '?' : '&') . 'wmode=transparent';

        return $url;
    }

    protected function getThumbTag($attributes)
    {
        $attributes->src   = $this->getThumbUrl();
        $attributes->class ??= 'video-thumbnail-youtube';

        Image::setAltAndTitle('video', $attributes);

        $image = new RL_Image($this->getThumbUrl(), $attributes);

        return $image->renderTag();
    }

    protected function getThumbUrl()
    {
        return 'https://img.youtube.com/vi/' . $this->id . '/mqdefault.jpg';
    }
}
