<?php
/**
 * @package         Articles Anywhere
 * @version         13.0.3PRO
 * 
 * @author          Peter van Westen <info@regularlabs.com>
 * @link            http://regularlabs.com
 * @copyright       Copyright © 2023 Regular Labs All Rights Reserved
 * @license         http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 */

namespace RegularLabs\Plugin\System\ArticlesAnywhere\Orderings;

defined('_JEXEC') or die;

use Joomla\Database\DatabaseQuery as JDatabaseQuery;
use RegularLabs\Library\ArrayHelper as RL_Array;
use RegularLabs\Plugin\System\ArticlesAnywhere\Helpers\Data as DataHelper;

class Orderings
{
    private array $orderings = [];

    /**
     * @param string $data
     * @param array  $params
     */
    public function __construct($ordering, $default_direction = 'ASC')
    {
        $this->setOrderings($ordering, $default_direction);
    }

    /**
     * @return Ordering[]
     */
    public function get()
    {
        return $this->orderings;
    }

    /**
     * @return array
     */
    public function getDatabaseStrings()
    {
        $orderings = [];

        foreach ($this->orderings as $ordering)
        {
            $orderings[] = $ordering->getDatabaseString();
        }

        return $orderings;
    }

    /**
     * @param JDatabaseQuery $query
     */
    public function setOnQuery($query)
    {
        foreach ($this->getDatabaseStrings() as $string)
        {
            $query->order($string);
        }
    }

    private function setOrderings($ordering, $default_direction = 'ASC')
    {
        $parts = RL_Array::toArray($ordering);

        foreach ($parts as $ordering)
        {
            if ($ordering === 'random')
            {
                $this->orderings[] = new Ordering('random');
                continue;
            }

            [$order, $direction] = RL_Array::toArray($ordering . ' ' . $default_direction, ' ');
            $data_group = DataHelper::getDataGroup($order);

            if ( ! $data_group)
            {
                continue;
            }

            $order = $data_group->getDatabaseKey();

            $this->orderings[] = new Ordering($order, $direction, $data_group);
        }
    }
}
